<!DOCTYPE html>
<html>
  <head>
    {% load static %}
    <meta charset="utf-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <title>Login to Django Log Lens</title>
    <meta name="viewport" content="width=device-width, initial-scale=1" />
    <link rel="stylesheet" type="text/css" media="screen" href='{% static "django_log_lens/styles.css" %}' />
    <link rel="icon" type="image/png" href="{% static 'django_log_lens/favicon.svg' %}" />
    <link rel="preconnect" href="https://fonts.googleapis.com" />
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin />
    <link
      href="https://fonts.googleapis.com/css2?family=Source+Code+Pro:ital,wght@0,200..900;1,200..900&display=swap"
      rel="stylesheet"
    />
  </head>

  <body style="background-color: var(--toolbar-color);">
      <div id="div-login" style="display: block; top: 10%">
        <br />
        <form method="post">
          {% csrf_token %}
          <img style="width: 50%" src="{% static 'django_log_lens/logo.svg' %}" alt="Django Log Lens" />
          <br />
          <br />
          <p class="error">{{ error_message }}</p>
          <label for="username">Username:</label>
          <input type="text" name="username" id="username" required />
          <br />
          <label for="password">Password:</label>
          <input type="password" name="password" id="password" required />
          <br />
          <br />
          <br />
          <button type="submit">Login</button>
        </form>
        <br />
    </div>
  </body>
</html>
